#!/usr/bin/env python2
# -*- coding: utf-8 -*-
##################################################
# GNU Radio Python Flow Graph
# Title: RTL HRIT Receiver
# Author: Raydel CM2ESP, based on original work from Martin Blaho
# Description: HRIT receiver
# Generated: Mon Jan  1 17:10:36 2018
##################################################

if __name__ == '__main__':
    import ctypes
    import sys
    if sys.platform.startswith('linux'):
        try:
            x11 = ctypes.cdll.LoadLibrary('libX11.so')
            x11.XInitThreads()
        except:
            print "Warning: failed to XInitThreads()"

from gnuradio import analog
from gnuradio import blocks
from gnuradio import digital
from gnuradio import eng_notation
from gnuradio import gr
from gnuradio import wxgui
from gnuradio.eng_option import eng_option
from gnuradio.fft import window
from gnuradio.filter import firdes
from gnuradio.wxgui import fftsink2
from gnuradio.wxgui import scopesink2
from grc_gnuradio import wxgui as grc_wxgui
from optparse import OptionParser
import osmosdr
import time
import wx


class hrit_rx_rtl(grc_wxgui.top_block_gui):

    def __init__(self, clock_alpha=1e-3, freq=1694100000, gainrf=48.0, pll_alpha=5e-3, samp_rate=1152e3, symb_rate=927000):
        grc_wxgui.top_block_gui.__init__(self, title="RTL HRIT Receiver ")
        _icon_path = "C:\Program Files\GNURadio-3.7\share\icons\hicolor\scalable/apps\gnuradio-grc.png"
        self.SetIcon(wx.Icon(_icon_path, wx.BITMAP_TYPE_ANY))

        ##################################################
        # Parameters
        ##################################################
        self.clock_alpha = clock_alpha
        self.freq = freq
        self.gainrf = gainrf
        self.pll_alpha = pll_alpha
        self.samp_rate = samp_rate
        self.symb_rate = symb_rate

        ##################################################
        # Variables
        ##################################################
        self.sps = sps = samp_rate*1.0/symb_rate

        ##################################################
        # Blocks
        ##################################################
        self.wxgui_scopesink2_0 = scopesink2.scope_sink_c(
        	self.GetWin(),
        	title='Scope Plot',
        	sample_rate=symb_rate,
        	v_scale=5,
        	v_offset=0,
        	t_scale=5,
        	ac_couple=False,
        	xy_mode=True,
        	num_inputs=1,
        	trig_mode=wxgui.TRIG_MODE_AUTO,
        	y_axis_label='Counts',
        )
        self.Add(self.wxgui_scopesink2_0.win)
        self.wxgui_fftsink2_0 = fftsink2.fft_sink_c(
        	self.GetWin(),
        	baseband_freq=freq,
        	y_per_div=10,
        	y_divs=10,
        	ref_level=0,
        	ref_scale=2.0,
        	sample_rate=samp_rate,
        	fft_size=1024,
        	fft_rate=5,
        	average=True,
        	avg_alpha=0.1,
        	title='FFT Plot',
        	peak_hold=False,
        	win=window.blackmanharris,
        )
        self.Add(self.wxgui_fftsink2_0.win)
        self.osmosdr_source_0 = osmosdr.source( args="numchan=" + str(1) + " " + 'rtl=0' )
        self.osmosdr_source_0.set_sample_rate(samp_rate)
        self.osmosdr_source_0.set_center_freq(freq, 0)
        self.osmosdr_source_0.set_freq_corr(0, 0)
        self.osmosdr_source_0.set_dc_offset_mode(0, 0)
        self.osmosdr_source_0.set_iq_balance_mode(0, 0)
        self.osmosdr_source_0.set_gain_mode(False, 0)
        self.osmosdr_source_0.set_gain(gainrf, 0)
        self.osmosdr_source_0.set_if_gain(20, 0)
        self.osmosdr_source_0.set_bb_gain(20, 0)
        self.osmosdr_source_0.set_antenna('', 0)
        self.osmosdr_source_0.set_bandwidth(0, 0)

        self.digital_costas_loop_cc_0 = digital.costas_loop_cc(pll_alpha, 2, False)
        self.digital_clock_recovery_mm_xx_0 = digital.clock_recovery_mm_cc(sps, clock_alpha**2/4.0, 0.5, clock_alpha, 0.005)
        self.digital_binary_slicer_fb_0 = digital.binary_slicer_fb()
        self.blocks_tcp_server_sink_0 = blocks.tcp_server_sink(gr.sizeof_char*1, '127.0.0.1', 8888, True)
        self.blocks_complex_to_real_0_0 = blocks.complex_to_real(1)
        self.analog_agc_xx_0 = analog.agc_cc(1e-3, 5, 1.0)
        self.analog_agc_xx_0.set_max_gain(65536)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.analog_agc_xx_0, 0), (self.digital_costas_loop_cc_0, 0))
        self.connect((self.analog_agc_xx_0, 0), (self.wxgui_fftsink2_0, 0))
        self.connect((self.blocks_complex_to_real_0_0, 0), (self.digital_binary_slicer_fb_0, 0))
        self.connect((self.digital_binary_slicer_fb_0, 0), (self.blocks_tcp_server_sink_0, 0))
        self.connect((self.digital_clock_recovery_mm_xx_0, 0), (self.blocks_complex_to_real_0_0, 0))
        self.connect((self.digital_clock_recovery_mm_xx_0, 0), (self.wxgui_scopesink2_0, 0))
        self.connect((self.digital_costas_loop_cc_0, 0), (self.digital_clock_recovery_mm_xx_0, 0))
        self.connect((self.osmosdr_source_0, 0), (self.analog_agc_xx_0, 0))

    def get_clock_alpha(self):
        return self.clock_alpha

    def set_clock_alpha(self, clock_alpha):
        self.clock_alpha = clock_alpha
        self.digital_clock_recovery_mm_xx_0.set_gain_omega(self.clock_alpha**2/4.0)
        self.digital_clock_recovery_mm_xx_0.set_gain_mu(self.clock_alpha)

    def get_freq(self):
        return self.freq

    def set_freq(self, freq):
        self.freq = freq
        self.wxgui_fftsink2_0.set_baseband_freq(self.freq)
        self.osmosdr_source_0.set_center_freq(self.freq, 0)

    def get_gainrf(self):
        return self.gainrf

    def set_gainrf(self, gainrf):
        self.gainrf = gainrf
        self.osmosdr_source_0.set_gain(self.gainrf, 0)

    def get_pll_alpha(self):
        return self.pll_alpha

    def set_pll_alpha(self, pll_alpha):
        self.pll_alpha = pll_alpha
        self.digital_costas_loop_cc_0.set_loop_bandwidth(self.pll_alpha)

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.set_sps(self.samp_rate*1.0/self.symb_rate)
        self.wxgui_fftsink2_0.set_sample_rate(self.samp_rate)
        self.osmosdr_source_0.set_sample_rate(self.samp_rate)

    def get_symb_rate(self):
        return self.symb_rate

    def set_symb_rate(self, symb_rate):
        self.symb_rate = symb_rate
        self.set_sps(self.samp_rate*1.0/self.symb_rate)
        self.wxgui_scopesink2_0.set_sample_rate(self.symb_rate)

    def get_sps(self):
        return self.sps

    def set_sps(self, sps):
        self.sps = sps
        self.digital_clock_recovery_mm_xx_0.set_omega(self.sps)


def argument_parser():
    description = 'HRIT receiver'
    parser = OptionParser(usage="%prog: [options]", option_class=eng_option, description=description)
    parser.add_option(
        "", "--clock-alpha", dest="clock_alpha", type="eng_float", default=eng_notation.num_to_str(1e-3),
        help="Set Clock alpha [default=%default]")
    parser.add_option(
        "", "--freq", dest="freq", type="intx", default=1694100000,
        help="Set Frequency [default=%default]")
    parser.add_option(
        "", "--gainrf", dest="gainrf", type="eng_float", default=eng_notation.num_to_str(48.0),
        help="Set Gain [default=%default]")
    parser.add_option(
        "", "--pll-alpha", dest="pll_alpha", type="eng_float", default=eng_notation.num_to_str(5e-3),
        help="Set PLL alpha [default=%default]")
    parser.add_option(
        "", "--samp-rate", dest="samp_rate", type="eng_float", default=eng_notation.num_to_str(1152e3),
        help="Set samp_rate [default=%default]")
    parser.add_option(
        "", "--symb-rate", dest="symb_rate", type="intx", default=927000,
        help="Set Symbol rate [default=%default]")
    return parser


def main(top_block_cls=hrit_rx_rtl, options=None):
    if options is None:
        options, _ = argument_parser().parse_args()

    tb = top_block_cls(clock_alpha=options.clock_alpha, freq=options.freq, gainrf=options.gainrf, pll_alpha=options.pll_alpha, samp_rate=options.samp_rate, symb_rate=options.symb_rate)
    tb.Start(True)
    tb.Wait()


if __name__ == '__main__':
    main()
